<?php

/*******************************************************************************
          ____                          ___       __          _     
         / __ \____ _      _____  _____/   | ____/ /___ ___  (_)___ 
        / /_/ / __ \ | /| / / _ \/ ___/ /| |/ __  / __ `__ \/ / __ \
       / ____/ /_/ / |/ |/ /  __/ /  / ___ / /_/ / / / / / / / / / /
      /_/    \____/|__/|__/\___/_/  /_/  |_\__,_/_/ /_/ /_/_/_/ /_/ 

*******************************************************************************/

/**
 * @file PowerAdmin - Domains Management
 */


/**
 * List domains
 * @return unknown_type
 */
function poweradmin_page_domains_list() {
  module_load_include('inc', 'poweradmin', 'poweradmin.api');
  $zones = poweradmin_list_domains();

  $header = array(t("Name"), t("Master"), t("Records"), t("Type"), t("Actions"));
  $rows = array();

  foreach ($zones as $zone) {
    $rows[] = array(
      $zone->name,
      $zone->master,
      $zone->count_records,
      $zone->type,
      _poweradmin_link_showzone($zone->id) .' '.
      _poweradmin_link_editzone($zone->id) .' '.
      _poweradmin_link_editrecords($zone->id) .' '.
      _poweradmin_link_delzone($zone->id),
    );
  }

  return theme('table', $header, $rows) . _poweradmin_link_addzone();
}

/**
 * List records associated to a domain
 * @param int $domain_id
 * @return string
 */
function poweradmin_page_show_domain($domain_id) {
  module_load_include('inc', 'poweradmin', 'poweradmin.api');
  $zone = poweradmin_get_domain($domain_id);
  drupal_set_title(t("Zone: %name", array("%name" => $zone->name)));

  
  $basic_info = array();
  
  $basic_info[] = array(
    array('data' => t("Type"), 'header' => TRUE, 'width' => '160'),
    array('data' => $zone->type),
  );
  if ($zone->master) {
    $basic_info .= " - <strong>". t("Master") .":</strong> ". $zone->master;
    $basic_info[] = array(
      array('data' => t("Master"), 'header' => TRUE, 'width' => '160'),
      array('data' => $zone->master),
    );
  }
  if ($zone->last_check) {
    #$basic_info .= " - <strong>". t("Last check") .":</strong> ". $zone->last_check;
    $basic_info[] = array(
      array('data' => t("Last check"), 'header' => TRUE),
      array('data' => $zone->last_check . " (". date("Y-m-d H:i:s", $zone->last_check) .")"),
    );
  }
  if ($zone->notified_serial) {
    #$basic_info .= " - <strong>". t("Notified serial") .":</strong> ". $zone->notified_serial;
    $basic_info[] = array(
      array('data' => t("Notified serial"), 'header' => TRUE),
      array('data' => $zone->notified_serial),
    );
  }
  
  $basic_info = theme('fieldset', array(
    '#title' => t("Zone definition"),
    '#value' => theme('table', array(), $basic_info),
    '#collapsible' => TRUE,
  ));
  
  
  if ($zone->soa) {
    $soa_rows = array();
    
    $soa_rows[] = array(
      array('data' => t("Primary"), 'header' => TRUE, 'width' => '160'),
      array('data' => $zone->soa['primary']),
    );
    $soa_rows[] = array(
      array('data' => t("Hostmaster"), 'header' => TRUE),
      array('data' => $zone->soa['hostmaster']),
    );
    $soa_rows[] = array(
      array('data' => t("Serial"), 'header' => TRUE),
      array('data' => $zone->soa['serial']),
    );
    $soa_rows[] = array(
      array('data' => t("Refresh"), 'header' => TRUE),
      array('data' => $zone->soa['refresh'] ." (". _poweradmin_timeint_format($zone->soa['refresh']) .")"),
    );
    $soa_rows[] = array(
      array('data' => t("Retry"), 'header' => TRUE),
      array('data' => $zone->soa['retry'] ." (". _poweradmin_timeint_format($zone->soa['retry']) .")"),
    );
    $soa_rows[] = array(
      array('data' => t("Expire"), 'header' => TRUE),
      array('data' => $zone->soa['expire'] ." (". _poweradmin_timeint_format($zone->soa['expire']) .")"),
    );
    $soa_rows[] = array(
      array('data' => t("Default TTL"), 'header' => TRUE),
      array('data' => $zone->soa['default_ttl'] ." (". _poweradmin_timeint_format($zone->soa['default_ttl']) .")"),
    );
    
    $soa_html = theme('table', array(), $soa_rows);
  }
  else {
    $soa_html = t("SOA not specified for this domain!");
  }
  
  $zone_info = theme('fieldset', array(
    '#title' => t("SOA Record"),
    '#value' => $soa_html,
    '#collapsible' => TRUE,
  ));

  #$zone_info = "<div>$zone_info</div>";

  $header = array(
    array('data' => t("Id"), 'width' => '40'),
    t("Name"),
    t("Type"),
    t("Content"),
    array('data' => t("Priority"), 'width' => '60'),
    array('data' => t("TTL"), 'width' => '60'),
    array('data' => t("Actions"), 'width' => '60'));
  $rows = array();

  foreach ($zone->records as $record_id => $record) {
    $rows[] = array(
      $record_id,
      $record->name,
      $record->type,
      $record->content,
      $record->prio,
      $record->ttl,
      _poweradmin_link_delrecord($record_id),
    );
  }
  
  $records_info = theme('fieldset', array(
    '#title' => t("Zone Records"),
    '#value' => theme('table', $header, $rows),
    '#collapsible' => TRUE,
  ));
  
  return $basic_info . $zone_info . $records_info ."<div>". _poweradmin_link_addrecord($domain_id) ."</div>";
}

/**
 * Form to edit the data related to a domain
 * @param unknown_type $form_state
 * @param unknown_type $domain_id
 * @return unknown_type
 */
function poweradmin_domain_edit_info_form($form_state, $domain_id = NULL) {
  module_load_include('inc', 'poweradmin', 'poweradmin.api');
  if ($domain_id) {
    $zone = poweradmin_get_domain($domain_id);
    drupal_set_title(t("Edit zone: %name", array("%name" => $zone->name)));
  }
  else {
    drupal_set_title(t("Create new zone"));
    // Prepare default values
    $zone = new stdClass();
    
  }
  
  // Create a default SOA record if not present in the domain
  if (!$zone->soa) {
    if ($domain_id) {
      drupal_set_message(t("No SOA was specified for the domain %name! Creating one now with default values.", array('%name' => $zone->name)), 'warning');
    }
    global $user;
    $zone->soa = array(
      'primary'     => $_SERVER['HTTP_HOST'],
      'hostmaster'  => $user->mail,
      'serial'      => "0",
      'refresh'     => "10800",
      'retry'       => "3600",
      'expire'      => "604800",
      'default_ttl' => "3600",
    );
  }

  $form = array();
  $form['domain_id'] = array(
    '#type' => 'value',
    '#value' => $domain_id,
  );
  $form['domain'] = array(
    '#type' => 'value',
    '#value' => $zone,
  );
  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t("Name"),
    '#default_value' => $zone->name,
    '#description' => t("The domain name for the zone."),
    '#required' => TRUE,
  );
  $form['type'] = array(
    '#type' => 'select',
    '#title' => t("Type"),
    '#default_value' => $zone->type,
    '#options' => _poweradmin_dtypes(),
    '#required' => TRUE,
    '#description' => t("The zone type"),
  );
  $form['master'] = array(
    '#type' => 'textfield',
    '#title' => t("Master"),
    '#default_value' => $zone->master,
    '#description' => t("The master nameserver name (for a slave zone)."),
  );
  $form['account'] = array(
    '#type' => 'textfield',
    '#title' => t("Account"),
    '#default_value' => $zone->account,
    '#description' => t(""),
  );
  
  $form['soa'] = array(
    '#type' => 'fieldset',
    '#title' => t("Start of Authority (SOA record)"),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,  
    '#tree' => TRUE,
  );
  $form['soa']['primary'] = array(
    '#type' => 'textfield',
    '#title' => t("Primary"),
    '#default_value' => $zone->soa['primary'],
    '#description' => t("the name of the master nameserver"),
    '#size' => 32,
    '#required' => TRUE,
  );
  $form['soa']['hostmaster'] = array(
    '#type' => 'textfield',
    '#title' => t("Hostmaster"),
    '#default_value' => $zone->soa['hostmaster'],
    '#description' => t("The hostmaster e-mail address"),
    '#size' => 32,
    '#required' => TRUE,
  );
  $form['soa']['serial'] = array(
    '#type' => 'textfield',
    '#title' => t("Serial"),
    '#default_value' => $zone->soa['serial'] ? $zone->soa['serial'] + 1 : 0,
    '#description' => t("The 'serial' field is special. If left at 0, the default, PDNS will perform an internal list of the domain to determine highest change_date field of all records within the zone, and use that as the zone serial number. This means that the serial number is always raised when changes are made to the zone, as long as the change_date field is being set."),
    '#size' => 8,
    '#required' => TRUE,
  );
  $form['soa']['refresh'] = array(
    '#type' => 'textfield',
    '#title' => t("Refresh"),
    '#default_value' => $zone->soa['refresh'],
    '#description' => t("The refresh time (in seconds)."),
    '#size' => 8,
    '#required' => TRUE,
  );
  $form['soa']['retry'] = array(
    '#type' => 'textfield',
    '#title' => t("Retry"),
    '#default_value' => $zone->soa['retry'],
    '#description' => t("The retry time (in seconds)."),
    '#size' => 8,
    '#required' => TRUE,
  );
  $form['soa']['expire'] = array(
    '#type' => 'textfield',
    '#title' => t("Expire"),
    '#default_value' => $zone->soa['expire'],
    '#description' => t("The expire time (in seconds)."),
    '#size' => 8,
    '#required' => TRUE,
  );
  $form['soa']['default_ttl'] = array(
    '#type' => 'textfield',
    '#title' => t("Default TTL"),
    '#default_value' => $zone->soa['default_ttl'],
    '#description' => t("The default TTL (in seconds)."),
    '#size' => 8,
    '#required' => TRUE,
  );
  
  $form['submit'] = array('#type' => 'submit', '#value' => t("Save Zone"));
  return $form;
}

/**
 * Domain info editing form - submit handler
 * Insert/update the domain information/SOA record
 * @param array $form
 * @param array $form_state
 * @return void
 */
function poweradmin_domain_edit_info_form_submit($form, &$form_state) {
  module_load_include('inc', 'poweradmin', 'poweradmin.api');
  $domain_id = $form_state['values']['domain_id'];
  $orig_domain = $form_state['values']['domain'];
  
  $definition = new stdClass();
  $definition->name = $form_state['values']['name'];
  $definition->type = $form_state['values']['type'];
  $definition->master = $form_state['values']['master'];
  $definition->last_check = $form_state['values']['last_check'];
  $definition->notified_serial = $form_state['values']['notified_serial'];
  
  if ($domain_id) {
    // Update zone
    poweradmin_update_domain($domain_id, $definition);
  }
  else {
    // Add zone
    poweradmin_insert_domain($definition);
    $domain_id = $definition->id;
  }

  // Update the SOA record for the zone
  // If a SOA record already exists, update it, else create one
  
  // Get the originale SOA record for the domain (if any)
  $soa_record = poweradmin_get_soa_record($domain_id);

  #dpm($form_state['values']);
  
  if (!$soa_record) {
    // If no SOA record found, create an empty one
    $soa_record = new stdClass();
    $soa_record->domain_id = $domain_id;
    $soa_record->type = 'SOA';
    $soa_record->ttl = $soa['default_ttl'];
    // TODO add SOA ttl to form fields
  }
  
  $soa = $form_state['values']['soa'];
  $soa_record->content = implode(' ', array(
    $soa['primary'],
    $soa['hostmaster'],
    $soa['serial'],
    $soa['refresh'],
    $soa['retry'],
    $soa['expire'],
    $soa['default_ttl'],
  ));
  
  // Insert/update the SOA record for this domain
  poweradmin_set_soa_record($domain_id, $soa_record);
  #dpm($soa_record, "\$soa_record");
  
  // Go to the zone page
  $form_state['redirect'] = "admin/poweradmin/zones/". $domain_id;
}

/**
 * Edit the records associated to a zone.
 * @param array $form_state
 * @param int $domain_id
 * @return array
 */
function poweradmin_domain_edit_records_form($form_state, $domain_id = NULL) {
  module_load_include('inc', 'poweradmin', 'poweradmin.api');
  $zone = poweradmin_get_domain($domain_id);
  
  if (!$zone) {
    // Zone not found. Return 404
    drupal_not_found();
    return;
  }
  
  drupal_set_title(t("Edit records for: %name", array("%name" => $zone->name)));

  $form = array();

  $form['zone'] = array('#type' => 'value', '#value' => $zone);
  $form['domain_id'] = array('#type' => 'value', '#value' => $domain_id);

  // TODO use the advanced_help module to show HTML help!
  $form['help'] = array(
    '#type' => 'fieldset',
    '#title' => t("Help"),
    '#value' => file_get_contents(drupal_get_path('module', 'poweradmin') .'/help/records-types.html'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  
  $form['records'] = array(
    '#type' => 'fieldset',
    '#tree' => TRUE,
    '#title' => t("Zone records"),
  );

  // Add fields for the records
  foreach ($zone->records as $record) {
    $form['records'][$record->id] = array(
      '#type' => 'fieldset',
    );
    $form['records'][$record->id]['id'] = array(
      '#type' => 'value',
      '#value' => $record->id,
    );
    $form['records'][$record->id]['name'] = array(
      '#type' => 'textfield',
      '#title' => t('Name'),
      '#default_value' => $record->name,
    );
    $form['records'][$record->id]['type'] = array(
      '#type' => 'select',
      '#title' => t('Type'),
      '#options' => _poweradmin_rtypes(),
      '#default_value' => $record->type,
    );
    $form['records'][$record->id]['content'] = array(
      '#type' => 'textfield',
      '#title' => t('Content'),
      '#default_value' => $record->content,
    );
    $form['records'][$record->id]['prio'] = array(
      '#type' => 'textfield',
      '#title' => t('Priority'),
      '#default_value' => $record->prio,
    );
    $form['records'][$record->id]['ttl'] = array(
      '#type' => 'textfield',
      '#title' => t('TTL'),
      '#default_value' => $record->ttl,
    );
    $form['records'][$record->id]['delete'] = array(
      '#type' => 'checkbox',
      '#title' => t('Delete'),
      '#default_value' => FALSE,
    );
  }

  for ($i=0; $i<10; $i++) {
    $form['records']["new-". $i] = array(
      '#type' => 'fieldset',
    );
    $form['records']["new-". $i]['id'] = array(
      '#type' => 'value',
      '#value' => NULL, // not already a record
    );
    $form['records']["new-". $i]['name'] = array(
      '#type' => 'textfield',
      '#title' => t('Name'),
    );
    $form['records']["new-". $i]['type'] = array(
      '#type' => 'select',
      '#title' => t('Type'),
      '#options' => _poweradmin_rtypes(),
    );
    $form['records']["new-". $i]['content'] = array(
      '#type' => 'textfield',
      '#title' => t('Content'),
    );
    $form['records']["new-". $i]['prio'] = array(
      '#type' => 'textfield',
      '#title' => t('Priority'),
    );
    $form['records']["new-". $i]['ttl'] = array(
      '#type' => 'textfield',
      '#title' => t('TTL'),
    );
    $form['records']["new-". $i]['delete'] = array(
      '#type' => 'checkbox',
      '#title' => t('Delete'),
      '#default_value' => FALSE,
    );
  }
  
  
  if ($domain_id) {
    $form['links'] = array(
      '#type' => 'markup',
      '#value' => _poweradmin_link_addrecord($domain_id),
      '#prefix' => '<div>',
      '#suffix' => '</div>',
    );
  }

  $form['submit'] = array('#type' => 'submit', '#value' => t("Save Records"));

  return $form;
}

/**
 * Edit zone submit handler
 * TODO rewrite this to update old records and insert new ones
 * @param array $form
 * @param array $form_state
 * @return void
 */
function poweradmin_domain_edit_records_form_submit($form, &$form_state) {
  module_load_include('inc', 'poweradmin', 'poweradmin.api');
  $domain_id = $form_state['values']['domain_id'];
  $zone = $form_state['values']['zone'];
  $cnt_update = 0;
  $cnt_insert = 0;
  $cnt_delete = 0;
  
  foreach (element_children($form_state['values']['records']) as $key) {
    $record_values = $form_state['values']['records'][$key];
    // If the 'delete' checkbox is set, delete the record (if id specified)
    if ($record_values['delete'] || !$record_values['content']) {
      if ($record_values['id']) {
        poweradmin_delete_record($record_values['id']);
        $cnt_delete++;
      }
    }
    // Update/insert the record
    else {
      $record = new stdClass();
      foreach (array('name', 'type', 'content', 'prio', 'ttl') as $varname) {
        $record->$varname = $record_values[$varname];
      }
      $record->domain_id = $domain_id;
      // If the 'id' field is set, update the selected record
      if ($record_values['id']) {
        $record->id = $record_values['id'];
        poweradmin_update_record($record_values['id'], $record);
        $cnt_update++;
      }
      // If no id specified, add a new record
      else {
        poweradmin_insert_record($record);
        $cnt_insert++;
      }
    }
  }
  
  // TODO Update the serial in the SOA for the zone
  //poweradmin_update_soa_serial($domain_id);
  
  // Set an info message
  drupal_set_message(t("Zone %name updated. %ci new records, %cu updated, %cd deleted", array(
    '%name' => $zone->name,
    '%ci' => $cnt_insert,
    '%cu' => $cnt_update,
    '%cd' => $cnt_delete,
  )));
}


/**
 * Delete zone form
 * @param array $form_state
 * @param int $domain_id
 * @return array
 */
function poweradmin_domain_delete_form($form_state, $domain_id) {
  module_load_include('inc', 'poweradmin', 'poweradmin.api');
  $zone = poweradmin_get_domain($domain_id);
  drupal_set_title(t("Delete zone: %name", array("%name" => $zone->name)));
  $form = array();
  $form['domain_id'] = array('#type' => 'value', '#value' => $domain_id);
  $form['message'] = array(
    '#type' => 'markup',
    '#value' => t("Are you sure to permanently delete zone %name?", array('%name' => $zone->name)),
    '#prefix' => '<div>', '#suffix' => '</div>');
  $form['confirm'] = array('#type' => 'submit', '#value' => t("Delete"));
  return $form;
}

/**
 * Delete zone submit handler
 * @param array $form
 * @param array $form_state
 * @return void
 */
function poweradmin_domain_delete_form_submit($form, &$form_state) {
  module_load_include('inc', 'poweradmin', 'poweradmin.api');
  $domain_id = $form_state['values']['domain_id'];
  $zone = poweradmin_get_domain($domain_id);
  poweradmin_delete_domain($domain_id);
  drupal_set_message(t("Deleted domain !name", array('!name' => $zone->name)));
  $form_state['redirect'] = 'admin/poweradmin/zones';
}

/**
 * Transform a form field #description into an image with tooltip text
 * @param array $element
 * @return void
 */
function _poweradmin_custom_help(&$element) {
  module_load_include('inc', 'poweradmin', 'poweradmin.api');
  
  //$description = htmlspecialchars($element['#description'], ENT_QUOTES);
  $description = $element['#description'];
  $help_icon = theme('image', _poweradmin_get_imagepath("help"), "[?]", $description);
  
  if ($description) {
    //$element['#title'] = $element['#title'] . " <a class='form-help' title='$description'>[?]</a>";
    $element['#title'] .= " $help_icon";
  }
  unset($element['#description']);
}

/**
 * Zone Informations form theming function 
 * @param array $form
 * @return string
 */
function theme_poweradmin_domain_edit_info_form($form) {
  _poweradmin_custom_help($form['name']);
  _poweradmin_custom_help($form['type']);
  _poweradmin_custom_help($form['master']);
  _poweradmin_custom_help($form['account']);
  _poweradmin_custom_help($form['soa']['primary']);
  _poweradmin_custom_help($form['soa']['hostmaster']);
  _poweradmin_custom_help($form['soa']['serial']);
  _poweradmin_custom_help($form['soa']['refresh']);
  _poweradmin_custom_help($form['soa']['retry']);
  _poweradmin_custom_help($form['soa']['expire']);
  _poweradmin_custom_help($form['soa']['default_ttl']);
  
  $form['name']['#attributes']['style'] = "width:100%;";
  $form['type']['#attributes']['style'] = "width:100%;";
  $form['master']['#attributes']['style'] = "width:100%;";
  $form['account']['#attributes']['style'] = "width:100%;";
  
  foreach (element_children($form['soa']) as $key) {
    $form['soa'][$key]['#attributes']['style'] = "width:100%;";
  }
  
  $output1 = "<table><tbody style='border:none;'>";
  $output1 .= "<tr><td colspan='10'>". drupal_render($form['name']) ."</td></tr>";
  $output1 .= "<tr>";
  $output1 .= "<td colspan='2' width='20%'>". drupal_render($form['type']) ."</td>";
  $output1 .= "<td colspan='6' width='60%'>". drupal_render($form['master']) ."</td>";
  $output1 .= "<td colspan='2' width='20%'>". drupal_render($form['account']) ."</td>";
  $output1 .= "</tr>";
  $output1 .= "</tbody></table>";
  
  $output2 = "<table><tbody style='border:none;'>";
  $output2 .= "<tr><td colspan='5' width='50%'>". drupal_render($form['soa']['primary']) ."</td><td colspan='5' width='50%'>". drupal_render($form['soa']['hostmaster']) ."</td></tr>";
  $output2 .= "<tr>";
  $output2 .= "<td colspan='2' width='20%'>". drupal_render($form['soa']['serial']) ."</td>";
  $output2 .= "<td colspan='2' width='20%'>". drupal_render($form['soa']['refresh']) ."</td>";
  $output2 .= "<td colspan='2' width='20%'>". drupal_render($form['soa']['retry']) ."</td>";
  $output2 .= "<td colspan='2' width='20%'>". drupal_render($form['soa']['expire']) ."</td>";
  $output2 .= "<td colspan='2' width='20%'>". drupal_render($form['soa']['default_ttl']) ."</td>";
  $output2 .= "</tr>";
  $output2 .= "</tbody></table>";
  
  $output .= theme('fieldset', array(
    '#title' => t("Basic informations"),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
    '#value' => "<div>$output1</div>",
  ));
  $output .= theme('fieldset', array(
    '#title' => $form['soa']['#title'],
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
    '#value' => "<div>$output2</div>",
  ));
  
  unset($form['soa']);
  
  return $output . drupal_render($form);
}

/**
 * Themeize the zone editing form
 * @param $zone
 * @return array
 */
function theme_poweradmin_domain_edit_records_form($form) {
  $header = array(
    array('data' => t("Id"), 'width' => '40'),
    t("Name"),
    t("Type"),
    t("Content"),
    array('data' => t("Priority"), 'width' => '60'),
    array('data' => t("TTL"), 'width' => '60'),
    array('data' => t("Delete"), 'width' => '60'),
    array('data' => t("Actions"), 'width' => '60'),
  );
  
  $rows = array();

  foreach (element_children($form['records']) as $domain_id) {
    $zone = $form['records'][$domain_id];
    $zone['name']['#size'] = 30;
    $zone['name']['#title'] = NULL;
    $zone['type']['#title'] = NULL;
    $zone['content']['#size'] = 30;
    $zone['content']['#title'] = NULL;
    $zone['prio']['#size'] = '8';
    $zone['prio']['#title'] = NULL;
    $zone['ttl']['#size'] = '10';
    $zone['ttl']['#title'] = NULL;
    $zone['delete']['#title'] = NULL;
    $rows[] = array(
      $domain_id,
      drupal_render($zone['name']),
      drupal_render($zone['type']),
      drupal_render($zone['content']),
      drupal_render($zone['prio']),
      drupal_render($zone['ttl']),
      drupal_render($zone['delete']),
      _poweradmin_link_delrecord($domain_id),
    );
  }

  unset($form['records']);
  return theme('table', $header, $rows) . drupal_render($form);
}
